-- | @HROOT@: a low-level binding to ROOT in haskell. 
--
--   For @ROOT@, please refer to <http://root.cern.ch>.
-- The following is a copy of its description in @ROOT@ homepage.
--
-- * The @ROOT@ system provides a set of OO frameworks with all the functionality needed to handle and analyze large amounts of data in a very efficient way. Having the data defined as a set of objects, specialized storage methods are used to get direct access to the separate attributes of the selected objects, without having to touch the bulk of the data. Included are histograming methods in an arbitrary number of dimensions, curve fitting, function evaluation, minimization, graphics and visualization classes to allow the easy setup of an analysis system that can query and process the data interactively or in batch mode, as well as a general parallel processing framework, @PROOF@, that can considerably speed up an analysis.
-- Thanks to the built-in @CINT@ C++ interpreter the command language, the scripting, or macro, language and the programming language are all C++. The interpreter allows for fast prototyping of the macros since it removes the, time consuming, compile/link cycle. It also provides a good environment to learn C++. If more performance is needed the interactively developed macros can be compiled using a C++ compiler via a machine independent transparent compiler interface called @ACliC@.
-- The system has been designed in such a way that it can query its databases in parallel on clusters of workstations or many-core machines. @ROOT@za is an open system that can be dynamically extended by linking external libraries. This makes @ROOT@ a premier platform on which to build data acquisition, simulation and data analysis systems.
-- @ROOT@ is available under the @LGPL@ license.
--
-- 
-- 
--   You can download @ROOT@ package and install in your local directory or 
--   in global directory. For local installation, 
--   You need to set environmental variable @ROOTSYS@ to the home directory
--   of ROOT and include @$ROOTSYS/bin@ and @$ROOTSYS/lib@ in your @PATH@ and 
--   @LD_LIBRARY_PATH@ or @DYLD_LIBRARY_PATH@ for @ROOT@ to work, respectively. 
--
--   HROOT package can be installed without ROOT, but it won't work. 
--   Examples are in example directory in the package.


module HROOT (
 module HROOT.Class.Deletable,  module HROOT.Class.TObject,  module HROOT.Class.TNamed,  module HROOT.Class.TDictionary,  module HROOT.Class.TClass,  module HROOT.Class.TFormula,  module HROOT.Class.TAtt3D,  module HROOT.Class.TAttAxis,  module HROOT.Class.TAttBBox,  module HROOT.Class.TAttCanvas,  module HROOT.Class.TAttFill,  module HROOT.Class.TAttImage,  module HROOT.Class.TAttLine,  module HROOT.Class.TAttMarker,  module HROOT.Class.TAttPad,  module HROOT.Class.TAttParticle,  module HROOT.Class.TAttText,  module HROOT.Class.THStack,  module HROOT.Class.TF1,  module HROOT.Class.TGraph,  module HROOT.Class.TGraphAsymmErrors,  module HROOT.Class.TCutG,  module HROOT.Class.TGraphBentErrors,  module HROOT.Class.TGraphErrors,  module HROOT.Class.TGraphPolar,  module HROOT.Class.TGraphQQ,  module HROOT.Class.TEllipse,  module HROOT.Class.TArc,  module HROOT.Class.TCrown,  module HROOT.Class.TLine,  module HROOT.Class.TArrow,  module HROOT.Class.TGaxis,  module HROOT.Class.TShape,  module HROOT.Class.TBRIK,  module HROOT.Class.TTUBE,  module HROOT.Class.TPCON,  module HROOT.Class.TSPHE,  module HROOT.Class.TXTRU,  module HROOT.Class.TBox,  module HROOT.Class.TPave,  module HROOT.Class.TPaveText,  module HROOT.Class.TDiamond,  module HROOT.Class.TPaveStats,  module HROOT.Class.TPavesText,  module HROOT.Class.TLegend,  module HROOT.Class.TLegendEntry,  module HROOT.Class.TPaveLabel,  module HROOT.Class.TPaveClass,  module HROOT.Class.TWbox,  module HROOT.Class.TFrame,  module HROOT.Class.TSliderBox,  module HROOT.Class.TTree,  module HROOT.Class.TChain,  module HROOT.Class.TNtuple,  module HROOT.Class.TNtupleD,  module HROOT.Class.TTreeSQL,  module HROOT.Class.TPolyLine,  module HROOT.Class.TCurlyLine,  module HROOT.Class.TCurlyArc,  module HROOT.Class.TEfficiency,  module HROOT.Class.TAxis,  module HROOT.Class.TLatex,  module HROOT.Class.TText,  module HROOT.Class.TDirectory,  module HROOT.Class.TDirectoryFile,  module HROOT.Class.TFile,  module HROOT.Class.TBranch,  module HROOT.Class.TVirtualTreePlayer,  module HROOT.Class.TTreePlayer,  module HROOT.Class.TArray,  module HROOT.Class.TArrayC,  module HROOT.Class.TArrayD,  module HROOT.Class.TArrayF,  module HROOT.Class.TArrayI,  module HROOT.Class.TArrayL,  module HROOT.Class.TArrayL64,  module HROOT.Class.TArrayS,  module HROOT.Class.TH1,  module HROOT.Class.TH2,  module HROOT.Class.TH3,  module HROOT.Class.TH1C,  module HROOT.Class.TH1D,  module HROOT.Class.TH1F,  module HROOT.Class.TH1I,  module HROOT.Class.TH1S,  module HROOT.Class.TH2C,  module HROOT.Class.TH2D,  module HROOT.Class.TH2F,  module HROOT.Class.TH2I,  module HROOT.Class.TH2Poly,  module HROOT.Class.TH2S,  module HROOT.Class.TH3C,  module HROOT.Class.TH3D,  module HROOT.Class.TH3F,  module HROOT.Class.TH3I,  module HROOT.Class.TH3S,  module HROOT.Class.TQObject,  module HROOT.Class.TVirtualPad,  module HROOT.Class.TPad,  module HROOT.Class.TButton,  module HROOT.Class.TGroupButton,  module HROOT.Class.TCanvas,  module HROOT.Class.TDialogCanvas,  module HROOT.Class.TInspectCanvas,  module HROOT.Class.TEvePad,  module HROOT.Class.TSlider,  module HROOT.Class.TApplication,  module HROOT.Class.TRint,  module HROOT.Class.TRandom,  module HROOT.Class.TCollection,  module HROOT.Class.TSeqCollection,  module HROOT.Class.TObjArray,  module HROOT.Class.TList,  module HROOT.Class.TKey,  module HROOT.Class.TDatime,  module HROOT.Class.TVirtualHistPainter 
) where

import HROOT.Class.Deletable
import HROOT.Class.TObject
import HROOT.Class.TNamed
import HROOT.Class.TDictionary
import HROOT.Class.TClass
import HROOT.Class.TFormula
import HROOT.Class.TAtt3D
import HROOT.Class.TAttAxis
import HROOT.Class.TAttBBox
import HROOT.Class.TAttCanvas
import HROOT.Class.TAttFill
import HROOT.Class.TAttImage
import HROOT.Class.TAttLine
import HROOT.Class.TAttMarker
import HROOT.Class.TAttPad
import HROOT.Class.TAttParticle
import HROOT.Class.TAttText
import HROOT.Class.THStack
import HROOT.Class.TF1
import HROOT.Class.TGraph
import HROOT.Class.TGraphAsymmErrors
import HROOT.Class.TCutG
import HROOT.Class.TGraphBentErrors
import HROOT.Class.TGraphErrors
import HROOT.Class.TGraphPolar
import HROOT.Class.TGraphQQ
import HROOT.Class.TEllipse
import HROOT.Class.TArc
import HROOT.Class.TCrown
import HROOT.Class.TLine
import HROOT.Class.TArrow
import HROOT.Class.TGaxis
import HROOT.Class.TShape
import HROOT.Class.TBRIK
import HROOT.Class.TTUBE
import HROOT.Class.TPCON
import HROOT.Class.TSPHE
import HROOT.Class.TXTRU
import HROOT.Class.TBox
import HROOT.Class.TPave
import HROOT.Class.TPaveText
import HROOT.Class.TDiamond
import HROOT.Class.TPaveStats
import HROOT.Class.TPavesText
import HROOT.Class.TLegend
import HROOT.Class.TLegendEntry
import HROOT.Class.TPaveLabel
import HROOT.Class.TPaveClass
import HROOT.Class.TWbox
import HROOT.Class.TFrame
import HROOT.Class.TSliderBox
import HROOT.Class.TTree
import HROOT.Class.TChain
import HROOT.Class.TNtuple
import HROOT.Class.TNtupleD
import HROOT.Class.TTreeSQL
import HROOT.Class.TPolyLine
import HROOT.Class.TCurlyLine
import HROOT.Class.TCurlyArc
import HROOT.Class.TEfficiency
import HROOT.Class.TAxis
import HROOT.Class.TLatex
import HROOT.Class.TText
import HROOT.Class.TDirectory
import HROOT.Class.TDirectoryFile
import HROOT.Class.TFile
import HROOT.Class.TBranch
import HROOT.Class.TVirtualTreePlayer
import HROOT.Class.TTreePlayer
import HROOT.Class.TArray
import HROOT.Class.TArrayC
import HROOT.Class.TArrayD
import HROOT.Class.TArrayF
import HROOT.Class.TArrayI
import HROOT.Class.TArrayL
import HROOT.Class.TArrayL64
import HROOT.Class.TArrayS
import HROOT.Class.TH1
import HROOT.Class.TH2
import HROOT.Class.TH3
import HROOT.Class.TH1C
import HROOT.Class.TH1D
import HROOT.Class.TH1F
import HROOT.Class.TH1I
import HROOT.Class.TH1S
import HROOT.Class.TH2C
import HROOT.Class.TH2D
import HROOT.Class.TH2F
import HROOT.Class.TH2I
import HROOT.Class.TH2Poly
import HROOT.Class.TH2S
import HROOT.Class.TH3C
import HROOT.Class.TH3D
import HROOT.Class.TH3F
import HROOT.Class.TH3I
import HROOT.Class.TH3S
import HROOT.Class.TQObject
import HROOT.Class.TVirtualPad
import HROOT.Class.TPad
import HROOT.Class.TButton
import HROOT.Class.TGroupButton
import HROOT.Class.TCanvas
import HROOT.Class.TDialogCanvas
import HROOT.Class.TInspectCanvas
import HROOT.Class.TEvePad
import HROOT.Class.TSlider
import HROOT.Class.TApplication
import HROOT.Class.TRint
import HROOT.Class.TRandom
import HROOT.Class.TCollection
import HROOT.Class.TSeqCollection
import HROOT.Class.TObjArray
import HROOT.Class.TList
import HROOT.Class.TKey
import HROOT.Class.TDatime
import HROOT.Class.TVirtualHistPainter

